class BaselineDifference < ActiveRecord::Base
  unloadable
  def self.per_page; 25 end

  belongs_to :node_baseline

  belongs_to :report_resource, :class_name => 'ResourceStatus'
  belongs_to :baseline_resource

  validate :accepted_xnor_decided_at
  def accepted_xnor_decided_at
    if accepted.nil?
      decided_at.nil? or errors.add :decided_at, "must not be set without accepted"
    else
      decided_at.nil? and errors.add :accepted, "must not be set without decided_at"
    end
  end

  named_scope :undecided, :conditions => 'accepted IS NULL'
  named_scope :accepted, :conditions => 'accepted = TRUE'
  named_scope :rejected, :conditions => 'accepted = FALSE'

  named_scope :scoped_by_node_id, lambda { |id|
    { :joins => :node_baseline, :conditions => { :node_baselines => { :node_id => id } } }
  }
  named_scope :self_comparisons, :joins => :node_baseline, :conditions => ['node_baselines.node_id = node_baselines.reference_node_id']
  named_scope :cross_comparisons, :joins => :node_baseline, :conditions => ['node_baselines.node_id <> node_baselines.reference_node_id']
  named_scope :scoped_by_group_id_and_date, lambda { |id, date|
    {
      :joins => { :node_baseline => { :node => :node_group_memberships } },
      :conditions => {
        :node_group_memberships => { :node_group_id => id },
        :node_baselines => { :date => date },
      },
    }
  }

  named_scope :filter_by_resource, lambda { |type, title|
    type  &&= type.gsub('%', '\%').gsub('_', '\_')
    title &&= title.gsub('%', '\%').gsub('_', '\_')

    {
      :conditions => ["resource_name LIKE ?", "%#{type}%[%#{title}%]"],
    }
  }

  named_scope :for_dates_other_than, lambda { |date|
    { :joins => :node_baseline, :conditions => ['date <> ?', date] }
  }

  named_scope :pending_changes_for_node_on_date, lambda {|node_id, date|
    start_date = BaselineUtil.day_beginning(date)
    {
      :conditions => ['accepted IS NOT NULL AND decided_at >= ? AND node_baselines.node_id = ?', start_date, node_id],
      :joins      => :node_baseline,
      :order      => :id,
    }
  }

  def self.can_change_all?(changes)
    count(:conditions => ["id IN (?) AND decided_at < ?", changes,
                        BaselineUtil.day_beginning(Date.today)]) == 0
  end

  def self.accept_all_changes(changes)
    return if changes.empty?
    return false unless can_change_all? changes
    transaction { changes.map &:accept_change }
  end

  def self.reject_all_changes(changes)
    return if changes.empty?
    return false unless can_change_all? changes
    transaction { changes.map &:reject_change }
  end

  def can_change?
    # If we have no decision, or it was decided inside the current window, we
    # can change our minds.  Otherwise ... no.
    if decided = decided_at then
      return decided > BaselineUtil.day_beginning(Date.today)
    end
    return true
  end

  def implement_change(to)
    return false unless can_change?
    update_attributes!(:accepted => to, :decided_at => Time.now)

    # This is recursive, because that hurts less than the alternatives...
    if nb = node_baseline.previous
      if diff = nb.differences.find_by_resource_name(resource_name)
        if diff.accepted.nil?
          if diff.resource_status_digest == resource_status_digest
            diff.implement_change(to)
          end
        end
      end
    end

    return true
  end

  def accept_change
    raise "Can't change this decision now" unless can_change?
    implement_change(true)
  end

  def reject_change
    raise "Can't change this decision now" unless can_change?
    implement_change(false)
  end

  def apply_decision_to(baseline)
    if accepted
      if old_resource = self.baseline_resource
        resource_to_replace = baseline.resources.find_by_resource_type_and_title(old_resource.resource_type, old_resource.title)
        if self.report_resource and resource_to_replace
          resource_to_replace.replace_with(self.report_resource)
        elsif self.report_resource
          baseline.resources << self.report_resource
        elsif resource_to_replace
          baseline.resources.delete(resource_to_replace)
          resource_to_replace.destroy
        end
      else
        new_resource = BaselineResource.create_from_resource_status(self.report_resource)
        baseline.resources << new_resource
      end
    end
  end

  def diffs(reload = false)
    if @diffs.nil? or reload
      r = {}
      report_resource and report_resource.events.each do |e|
        r[e.property] = e.previous_value
      end

      b = {}
      baseline_resource and baseline_resource.properties.each do |e|
        b[e.name] = e.value
      end

      @diffs = (r.keys | b.keys).sort_by do |x|
        [x == "ensure" ? 0 : 1, x]
      end.collect do |key|
        { :property => key, :actual => r[key], :baseline => b[key] }
      end
    end

    @diffs
  end
end
